<?php


class WC_Bws_Captcha {

	static protected $error_message;
	static protected $label_text;
	static protected $plugin_options;

	public static function initialize() {

		self::$plugin_options = get_option( 'wc_bws_options' );
		self::$error_message = self::$plugin_options['error_message'];
		self::$label_text = self::$plugin_options['label_text'];

	}

	/** Output the BestWebSoft CAPTCHA form field. */
	public static function display_captcha() {

		$wc_bws_options = get_option( 'wc_bws_options' );
		$label_text = isset( $wc_bws_options['label_text'] ) ? $wc_bws_options['label_text'] : '';

		echo '<div class="wc-captcha">';
		
		if ($label_text != '') {
            echo '<label for="captcha">';
            _e( $label_text . ' ', 'wc-no-captcha' ); 
            echo '<span class="required">*</span></label>';
        }
	    
	    
        if( function_exists( 'cptch_display_captcha_custom' ) ) { 
           echo "<input type='hidden' name='cntctfrm_contact_action' value='true' />"; 
           echo cptch_display_captcha_custom(); 
        } 
        elseif( function_exists( 'cptchpr_display_captcha_custom' ) ) { 
           echo "<input type='hidden' name='cntctfrm_contact_action' value='true' />"; 
           echo cptchpr_display_captcha_custom(); 
        } 
        else {
           echo "A compatible version of Captcha by BestWebSoft is not installed. "; 
           echo "Please deactivate your installed Captcha plugin in WordPress. ";
           echo "This will enable the compatible version bundled with this plugin. "; 
        }
        
        echo '</div>';
	}


	public static function on_activation() {
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}
		$plugin = isset( $_REQUEST['plugin'] ) ? $_REQUEST['plugin'] : '';
		check_admin_referer( "activate-plugin_{$plugin}" );
		
		$default_options = array(
			'error_message'     => __('<strong>ERROR</strong>: Your arithmetic was incorrect.', 'wc-bws-captcha'),
			'label_text'        => __('CAPTCHA Arithmetic Test', 'wc-bws-captcha')
		);
		
        $wc_bws_options = get_option( 'wc_bws_options' );
        if ($wc_bws_options == false) {
            add_option( 'wc_bws_options', $default_options );
        } else {
            if (! isset( $wc_bws_options['error_message'] ) ) { $wc_bws_options['error_message'] = $default_options['error_message'] ; }
            if (! isset( $wc_bws_options['label_text'] ) ) { $wc_bws_options['label_text'] = $default_options['label_text'] ; }
            update_option( 'wc_bws_options', $wc_bws_options );
        }
            
	}

	public static function on_uninstall() {
		if ( ! current_user_can( 'activate_plugins' ) ) {
			return;
		}

		delete_option( 'wc_bws_options' );
	}
}
