<?php
ob_start();

class WC_Bws_Settings_Page {

	public static function initialize() {
		add_action( 'admin_menu', array( __CLASS__, 'register_menu_page' ) );
	}


	public static function register_menu_page() {

		add_submenu_page(
			'woocommerce',
			__( 'Captcha for WooCommerce', 'woocommerce' ),
			__( 'Captcha', 'woocommerce' ),
			'manage_options',
			'wc-bws',
			array(
				__CLASS__,
				'settings_page',
			),
			null
		);

	}

	public static function settings_page() {

		$wc_bws_options = get_option( 'wc_bws_options' );

		$captcha_wc_login        = isset( $wc_bws_options['captcha_wc_login'] ) ? $wc_bws_options['captcha_wc_login'] : '';
		$captcha_wc_registration = isset( $wc_bws_options['captcha_wc_registration'] ) ? $wc_bws_options['captcha_wc_registration'] : '';
		$captcha_wc_lost_password = isset( $wc_bws_options['captcha_wc_lost_password'] ) ? $wc_bws_options['captcha_wc_lost_password'] : '';
		$captcha_wc_checkout     = isset( $wc_bws_options['captcha_wc_checkout'] ) ? $wc_bws_options['captcha_wc_checkout'] : '';
		$error_message = isset( $wc_bws_options['error_message'] ) ? $wc_bws_options['error_message'] : '';
		$label_text = isset( $wc_bws_options['label_text'] ) ? $wc_bws_options['label_text'] : '';
		$embedded_message = isset( $wc_bws_options['embedded_message'] ) ? $wc_bws_options['embedded_message'] : '';

		// call to save the setting options
		self::save_options();
		?>
		<style>
			input[type='text'], textarea, select {
				width: 600px;
			}
		</style>
		<div class="wrap">

			<div id="icon-options-general" class="icon32"></div>
			<h2><?php _e('Captcha for WooCommerce', 'wc-bws-captcha'); ?></h2>

			<p><?php _e('Protect WooCommerce login registration and lost password form against spam using Captcha by BestWebSoft.', 'wc-bws-captcha'); ?></p>

			<?php
			if ( isset( $_GET['settings-updated'] ) && ( $_GET['settings-updated'] ) ) {
				echo '<div id="message" class="updated"><p><strong>' . __('Settings saved.', 'wc-bws-captcha') . '</strong></p></div>';
			}
			?>
			<div id="poststuff">

				<div id="post-body" class="metabox-holder columns-2">

					<!-- main content -->
					<div id="post-body-content">

						<div class="meta-box-sortables ui-sortable">

							<form method="post">

								<div class="postbox">

									<div title="Click to toggle" class="handlediv"><br></div>
									<h3 class="hndle"><span><?php _e( 'Display Settings', 'wc-bws-captcha' ); ?></span>
									</h3>

									<div class="inside">
										<table class="form-table">
											<tr>
												<th scope="row">
													<label for="login"><?php _e( 'Login Form', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="login" type="checkbox" name="wc_bws_options[captcha_wc_login]"
													       value="yes" <?php checked( $captcha_wc_login, 'yes' ) ?>>

													<p class="description">
														<?php _e( 'Check to enable CAPTCHA in WooCommerce login form', 'wc-bws-captcha' ); ?>

													</p>
												</td>
											</tr>
											<tr>
												<th scope="row"><label
														for="registration"><?php _e( 'Registration Form', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="registration" type="checkbox" name="wc_bws_options[captcha_wc_registration]"
													       value="yes" <?php checked( $captcha_wc_registration, 'yes' ) ?>>

													<p class="description">
														<?php _e( 'Check to enable CAPTCHA in WooCommerce registration form', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
											<tr>
												<th scope="row"><label
														for="lost_password"><?php _e( 'Lost Password Form', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="lost_password" type="checkbox" name="wc_bws_options[captcha_wc_lost_password]"
													       value="yes" <?php checked( $captcha_wc_lost_password, 'yes' ) ?>>

													<p class="description">
														<?php _e( 'Check to enable CAPTCHA in WooCommerce lost password form', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
											<tr>
												<th scope="row"><label
														for="checkout"><?php _e( 'Checkout Payment Form', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="checkout" type="checkbox" name="wc_bws_options[captcha_wc_checkout]"
													       value="yes" <?php checked( $captcha_wc_checkout, 'yes' ) ?>>

													<p class="description">
														<?php _e( 'Check to enable CAPTCHA in WooCommerce checkout payment form', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
										</table>
										<p>
											<?php wp_nonce_field( 'wc_bws_settings_nonce' ); ?>
											<input class="button-primary" type="submit" name="settings_submit"
											       value="Save All Changes">
										</p>
									</div>
								</div>


								<div class="postbox">

									<div class="handlediv"><br></div>
									<h3 class="hndle"><span><?php _e( 'General Settings', 'wc-bws-captcha' ); ?></span>
									</h3>

									<div class="inside">

										<table class="form-table">
											<tr>
												<th scope="row"><label
														for="message"><?php _e( 'Error Message', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="message" type="text" name="wc_bws_options[error_message]"
													       value="<?php echo $error_message; ?>">

													<p class="description">
														<?php _e( 'Message or text to display when CAPTCHA is ignored or the test has failed.', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
											<tr>
												<th scope="row"><label
														for="label"><?php _e( 'Label Text', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="label" type="text" name="wc_bws_options[label_text]"
													       value="<?php echo $label_text; ?>">

													<p class="description">
														<?php _e( 'Label text to display for the CAPTCHA test.', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
										</table>
										<p>
											<?php wp_nonce_field( 'settings_nonce' ); ?>
											<input class="button-primary" type="submit" name="settings_submit"
											       value="Save All Changes">
										</p>
									</div>
								</div>

								<div class="postbox">

									<div class="handlediv"><br></div>
									<h3 class="hndle"><span><?php _e( 'Captcha by BestWebSoft', 'wc-bws-captcha' ); ?></span>
									</h3>

									<div class="inside">

										<table class="form-table">
											<tr>
												<th scope="row"><label
														for="embedded"><?php _e( 'Using Embedded Version', 'wc-bws-captcha' ); ?></label>
												</th>
												<td>
													<input id="embedded" type="text" name="wc_bws_options[embedded_message]" readonly="readonly"
													       value="<?php echo $embedded_message; ?>">

													<p class="description">
														<?php _e( 'Indicates if embedded Captcha by BestWebSoft version is being used.', 'wc-bws-captcha' ); ?>
													</p>
												</td>
											</tr>
										</table>
									</div>
								</div>

							</form>
						</div>
					</div>
					<div id="postbox-container-1" class="postbox-container">

						<div class="meta-box-sortables">

							<div class="postbox">
								<div class="handlediv"><br></div>
								<h3 class="hndle" style="text-align: center;">
									<span><?php _e( 'Developer', 'wc-bws-captcha' ); ?></span>
								</h3>

								<div class="inside">
									<div style="text-align: center; margin: auto">Adapted for BestWebSoft Captcha by <br> <a
											href="https://bronzeart.ca/captcha-for-woocommerce/"><strong>William Miles</strong></a></div>

									<div style="text-align: center; margin: auto">Made with lots of love by <br> <a
											href="http://w3guy.com"><strong>Agbonghama Collins</strong></a></div>

									<div style="text-align: center; margin: auto">Thanks to BestWebSoft Captcha by <br> <a
											href="https://bestwebsoft.com/"><strong>BestWebSoft</strong></a></div>

								</div>
							</div>


							<div class="postbox">
								<div class="handlediv"><br></div>
								<h3 class="hndle" style="text-align: center;">
									<span><?php _e( 'Support Plugin', 'wc-bws-captcha' ); ?></span>
								</h3>

								<div class="inside">
									<div style="text-align: center; margin: auto">
										<p>
											Is this plugin useful for you? If so, please help support my work with a
											<a href="https://www.bronzeart.ca/shop/donation/" target="_blank">donation</a>.
										</p>
									</div>
								</div>
							</div>
						</div>
					</div>
				</div>
				<br class="clear">
			</div>
		</div>
	<?php
	}


	public static function save_options() {
		if ( isset( $_POST['settings_submit'] ) && check_admin_referer( 'settings_nonce', '_wpnonce' ) ) {

			$saved_options = $_POST['wc_bws_options'];

			update_option( 'wc_bws_options', $saved_options );

			wp_redirect( '?page=wc-bws&settings-updated=true' );
			exit;
		}
	}
}

ob_clean();
